/***********************************************************************
 * Macro qui assigne les deux dimensions de l'indice de           ******
 * dfavorisation qubcois de 1991 et la zone  partir du        ******
 * code postal et un code municipal  tout fichier administratif  ******
 * comprenant au moins le code postal. On utilise une version     ******
 * simplifie du FCCP+ produit par Statistique Canada.            ******
 * 																  ******
 * Paramtres de la macro                                         ******
 *  - IN: fichier SAS d'entre auquel on veut ajouter l'indice    ******
 *        de dfavorisation (matriel et social)                  ******
 *  - PCODE: nom de la variable du code postal  6 positions      ******
 *  - MUNIC: nom de la variable du code municipal. Par dfaut,    ******
 *           0=l'attribution ne se fait qu' partir du code       ******
 *           postal seulement                                     ******
 *  - OUT: nom du fichier SAS de sortie identique  IN mais les   ******
 *         nouvelles variables et les indices de dfavorisation   ******
 * Auteur: Philippe Gamache, INSPQ                                ******
 * Dernire modification: 22 avril 2020                           ******
 * Modifications: Ajout des quintiles rgionaux			          ******
 *                Renouvellement du fichier munic			      ******
 ***********************************************************************/

*** remplacer les chemins d'accs aux fichiers selon vos emplacements ***;
libname i1991 "E:\Projets\IndiceDfavorisation\Indice1991\Assignation\Qubec";

filename fccp "E:\Projets\IndiceDfavorisation\Indice1991\Assignation\Qubec\fccp.txt";
filename munic "E:\Projets\IndiceDfavorisation\Indice1991\Assignation\Qubec\munic2016.txt";
filename dec04 "E:\Projets\IndiceDfavorisation\Indice1991\Assignation\Qubec\dec04.txt"; /*exemple de fichier texte contenant des codes postaux */


*** lire un fichier texte avec SAS ***;
data i1991.dec04;
infile dec04;
input  cp $ 1-6 cod_mun 7;
run;

data i1991.fccp;
infile fccp;
input  munic 1-12 pcode $ 13-18 @19 propcp 5.2 propmunic 5.2 QuintMat 33 QuintSoc 34 CentMat 35-37 CentSoc 38-40 zone 41
       RSS 42-43 QuartMatRSS 44 QuartSocRSS 45 QuintMatRSS 46 QuintSocRSS 47;
run;

data i1991.munic;
  infile munic;
  input munic 1-7 mun1991 8-14;
run;

%macro presence(var=);
%global presence;
%let dsid=%sysfunc(open(fichin,i));
    %if &dsid>0 %then
       %do;
	     %let presence=%sysfunc(varnum(&dsid,&var));
		 /*** la macrovariable presence = 0 si la variable est absente du fichier ****/
	   %end;
%put &presence;
%let rc = %sysfunc(close(&dsid));
%mend presence;


%macro Assignation(in,pcode,munic,out);

data fccp;
   set i1991.fccp;
   nuniq = _n_;
run;

proc sort data=i1991.munic;by munic;run;
proc sort data=fccp;by munic;run;

data fccp;
  merge i1991.munic(in=in2) fccp(in=in1);
  by munic;
  if in1*in2 then munic=mun1991;
  else if in2 then delete;
run;

data fichin;
  %if &munic ne 0 %then %do; length munic 8.; %end;
  set &in;
  numero=_n_;
  nba=ranuni(766);
  pcode=compress(&pcode);
  %if &munic ne 0 %then %do; 
    munic=2400000+mod(&munic,2400000); 
  %end;
run;

%presence(var=quintmat);

%if &presence ne 0 %then %do;
  data fichin;
  set fichin;
  where zone in (.,0);
  run;
%end;

data fichin;
  set fichin;
  keep numero nba pcode %if &munic ne 0 %then %do; munic %end;;
run;

%if &munic ne 0 %then %do;


proc sort data=fichin;by munic;run;

data fichin;
  merge i1991.munic(in=in2) fichin(in=in1);
  by munic;
  if in1*in2 then munic=mun1991;
  else if in2 then delete;
run;

proc sql;
  create table test as
  select fichin.pcode, fichin.munic, numero, nba, fccp.propmunic, fccp.nuniq, fccp.quintmat, fccp.quintsoc, fccp.centmat, fccp.centsoc, 
		 fccp.zone, fccp.rss, fccp.quartmatrss, fccp.quartsocrss, fccp.quintmatrss, fccp.quintsocrss
  from fichin, fccp
  where fichin.pcode=fccp.pcode and fichin.munic=fccp.munic
  order by numero,nuniq; 
quit;

proc sort data=test;by pcode munic numero;run;

/*** Choix de l'indice de dfavorisation si plus d'un indice par code postal ***/
data test2;
  set test;
  by pcode munic numero;
  retain cumul find direct;
  if first.numero then do; 
    find=0; 
    cumul=0; 
    if last.numero then direct=1; 
                   else direct=0;
  end;
  if find = 0 then do;
    cumul=cumul+(propmunic/100);
	if nba < cumul then do; 
      find=1; 
      output; 
    end;
  end;
run; 

proc sort data=test2;by numero;run;
proc sort data=fichin;by numero;run; 

data appmunic1 fichin;
  merge fichin(in=in1) test2(in=in2);
  by numero;
  if in1*in2 then output appmunic1;
  else if in1 then output fichin;
run;

%end;

proc sql;
  create table test as
  select fichin.pcode, numero, nba, fccp.propcp, fccp.nuniq, fccp.quintmat, fccp.quintsoc, fccp.centmat, fccp.centsoc, 
         fccp.zone, fccp.rss, fccp.quartmatrss, fccp.quartsocrss, fccp.quintmatrss, fccp.quintsocrss
  from fichin, fccp
  where fichin.pcode=fccp.pcode
  order by numero,nuniq; 
quit;

proc sort data=test;by pcode numero;run;

/*** Choix de l'indice de dfavorisation si plus d'un indice par code postal ***/
data test2;
  set test;
  by pcode numero;
  retain cumul find direct;
  if first.numero then do; 
    find=0; cumul=0; 
    if last.numero then direct=1; 
                   else direct=0;
  end;
  if find = 0 then do;
    cumul=cumul+(propcp/100);
	if nba < cumul then do; 
      find=1; 
      output; 
    end;
  end;
run; 

proc sort data=test2;by numero;run;
proc sort data=fichin;by numero;run; 

data appcp1 noapp1;
  merge fichin(in=in1) test2(in=in2);
  by numero;
  if in1*in2 then output appcp1;
  else if in1 then output noapp1;
run;

data tout;
  set %if &munic ne 0 %then %do; appmunic1(in=in1)%end;
  appcp1(in=in2) noapp1(in=in3);
  if in3=1 then do; 
    quintmat = 0; quintsoc = 0; 
    centmat = 0;centsoc = 0; 
    zone = 0;
    quintmatrss=0;quintsocrss=0;
	quartmatrss=0;quartsocrss=0;
  end;

  if quintmat=0 then groupe=3;
  else if quintmat=. then groupe=2;
  else groupe=1;
  keep numero groupe centmat centsoc quintmat quintsoc quartmatRSS quartsocRSS quintmatrss quintsocrss rss zone;
run;

proc sort; by numero; run; 

data fichin;
  set &in;
  numero=_n_;
run;

data tout;
  retain CentMat CentSoc QuintMat QuintSoc QuintMatRSS QuintSocRSS QuartMatRSS QuartSocRSS RSS zone groupe numero;
  set tout;
run;

data &out;
  merge fichin tout;
  by numero;
  Date = '20200422';
  format groupe fgroupe.;
  drop numero;
run;

proc datasets library=work;
  delete pccfuniq2 pccfdups2 test test2 fichin tout appmunic1 noapp1; 
quit;
%mend Assignation;

proc format;
  value fgroupe 1="Assignation russie"
  		        2="Appartient  un SD sans indice"
				3="Aucun appariement CP - SD - Indice";
run;

/****************************************************************
* Exemple: %assignation(dec04, cp, cod_mun, dec04v2); 	    	*
* - Le fichier en entre est WORK.dec04              			*
* - La variable pour le code postal est CP             			*
* - La variable pour le code municipal est COD_MUN     			*
*   N.B.: on doit mettre 0 (zro) si on ne veut pas faire   	*
*   l'assignation  l'aide du code municipal     				*
* - Le fichier rsultant est WORK.dec04v2. Il est identique 	*
*   au fichier d'entre, mais avec l'indice de dfavorisation  	*
*   en quintiles, en quartiles, la rgion et la variable zone  	*
* - NOTE: Les rsultats peuvent varier si l'ordre du fichier  	*
*   d'entre n'est pas la mme.                					*
****************************************************************/ 

%Assignation(in=i1991.dec04,pcode=cp,munic=cod_mun,out=i1991.dec04v2); /* exemple de commande qui va utiliser la macro, voir le Guide d'utilisateur */

proc export 
  data=i1991.dec04v2
  dbms=xlsx
  outfile="C:\MyData\i1991.dec04v2.xlsx"
  replace;
run;
